/* SPDX-License-Identifier: GPL-2.0 */
#ifndef __ASM_MACH_LOONGSON64_BOOT_PARAM_H_
#define __ASM_MACH_LOONGSON64_BOOT_PARAM_H_


#define ADDRESS_TYPE_SYSRAM	1
#define ADDRESS_TYPE_RESERVED	2
#define ADDRESS_TYPE_ACPI	3
#define ADDRESS_TYPE_NVS	4
#define ADDRESS_TYPE_PMEM	5

#define LOONGSON3_BOOT_MEM_MAP_MAX 128

struct loongson_board_info {
	int bios_size;
	char *bios_vendor;
	char *bios_version;
	char *bios_release_date;
	char *board_name;
	char *board_vendor;

};

struct loongson_system_configuration {
	char *cpuname;
	int nr_cpus;
	int nr_nodes;
	int cores_per_node;
	int cores_per_package;
	u16 boot_cpu_id;
	u64 reserved_cpus_mask;
	u64 ht_control_base;
	u64 restart_addr;
	u64 poweroff_addr;
	u64 suspend_addr;
	u64 vgabios_addr;
	u32 dma_mask_bits;
	u32 msi_address_lo;
	u32 msi_address_hi;
	u32 msi_base_irq;
	u32 msi_last_irq;
	u32 io_base_irq;
	u32 io_last_irq;
	u8 pcie_wake_enabled;
};


#define LOONGSON_DMA_MASK_BIT		64
#define LOONGSON_MEM_LINKLIST		"MEM"
#define LOONGSON_VBIOS_LINKLIST		"VBIOS"
#define LOONGSON_EFIBOOT_SIGNATURE	"BPI"

struct bootparamsinterface {
	u64	signature;	/*{"B", "P", "I", "_", "0", "_", "1"}*/
	void	*systemtable;
	struct	_extention_list_hdr	*extlist;
}__attribute__((packed));

struct _extention_list_hdr {
	u64	signature;
	u32	length;
	u8	revision;
	u8	checksum;
	struct	_extention_list_hdr *next;
}__attribute__((packed));

struct loongsonlist_mem_map {
	struct	_extention_list_hdr header;	/*{"M", "E", "M"}*/
	u8	map_count;
	struct	_loongson_mem_map {
		u32 mem_type;
		u64 mem_start;
		u64 mem_size;
	}__attribute__((packed))map[LOONGSON3_BOOT_MEM_MAP_MAX];
}__attribute__((packed));

struct loongsonlist_vbios {
	struct	_extention_list_hdr header;	/* {VBIOS} */
	u64	vbios_addr;
}__attribute__((packed));
extern void *loongson_fdt_blob;
extern u32 __dtb_loongson3_ls7a_begin[];
extern struct loongson_board_info b_info;
extern struct loongson_system_configuration loongson_sysconf;
extern struct bootparamsinterface *efi_bp;
#endif
